<?php
/**
 * Default copyright laws apply.
 *
 * Veebipoed retains all rights to this source code and nobody else may reproduce, distribute
 * or create derivative works from this work.
 *
 * Permitted:
 * - Commercial Use
 * This software and derivatives may be used for commercial purposes.
 * - Private Use
 * You may use and modify the software without distributing it.
 *
 * Forbidden:
 * - Distribution
 * You may not distribute this software.
 * - Modification
 * This software may not be modified.
 * - Sublicensing
 * You may not grant a sublicense to modify and distribute this software to third parties not included in the license.
 *
 *  @author    Veebipoed.ee
 *  @copyright 2020 Veebipoed.ee
 *  @license   Veebipoed.ee
 */

class lhvleaseconfirmationModuleFrontController extends ModuleFrontController
{
    private $server = null;

    public function initContent()
    {
        parent::initContent();
        $this->server = $this->module->getConfig('server');
        $this->context->smarty->assign(array(
            'formdata' => $this->htmlspecialcharsAdvanced($this->getFormFields()),
            'banklink_msg' => trim(
                Configuration::get(
                    $this->module->getConfig('payment_confirm'),
                    $this->context->language->id
                )
            ),
            'bank_url' => $this->module->getConfigFromFile($this->server, 'url'),
        ));

        $this->setTemplate('module:lhvlease/views/templates/front/confirmation.tpl');
    }

    private function htmlspecialcharsAdvanced($fields)
    {
        $fieldsOriginal = ['VK_DATA', 'VK_MAC'];

        foreach ($fields as $field => &$value) {
            if (!in_array($field, $fieldsOriginal)) {
                htmlspecialchars($value);
            }
        }

        return $fields;
    }

    private function createOrderXML()
    {
        $cart_rules = $this->context->cart->getCartRules();
        $currency = Currency::getCurrency($this->context->cart->id_currency);
        $discount = 0;
        $highest_product_price = 0;
        $products = $this->context->cart->getProducts();
        $most_expensive_product_pos = null;
        $total = $this->context->cart->getOrderTotal(true, Cart::ONLY_PRODUCTS);
        $xml = new SimpleXMLElement('<CofContractProductList></CofContractProductList>');

        foreach ($cart_rules as $cart_rule) {
            $discount += $cart_rule['value_real'];
        }
        $remaining_discount = $discount = Tools::ps_round($discount, 2);


        foreach ($products as $i => &$productArr) {
            $productArr['price_wt'] = Tools::ps_round($productArr['price_wt'], 2);

            if ($discount == 0) {
                $productArr['discount'] = 0;
                continue;
            }

            if ($productArr['price_wt'] > $highest_product_price) {
                $most_expensive_product_pos = $i;
                $highest_product_price = $productArr['price_wt'];
            }

            $productArr['discount'] = Tools::ps_round($productArr['price_wt'] / $total * $discount, 2);
            $remaining_discount -= ($productArr['discount'] * $productArr['cart_quantity']);
        }
        unset($productArr);

        $remaining_discount = Tools::ps_round($remaining_discount, 2);

        /**
         * If we have more than 6 ( 7 without shipping) products
         * we need to cut them down and add them to the last product.
         * Because VK_DATA limit is 4096 chars. 1 product takes up maximum of 580 chars.
         * @version 3.0.1
         */

        $total_products = count($products);
        $shipping_included = $this->module->getConfig('with_shipping');
        $cut_size = $shipping_included ? 6 : 7;

        $left_overs = [];

        if ($total_products > $cut_size) {
            $left_overs = array_slice($products, $cut_size - 1);
            $products = array_slice($products, 0, $cut_size - 1);
        }

        $product_count = 0;

        foreach ($products as $j => $productArr) {
            $name = $productArr['name'].(empty($productArr['attributes']) ? '' : ' - '.$productArr['attributes']);

            for ($i = 0; $i < $productArr['cart_quantity']; $i++) {
                $product_count++;

                $discount = $productArr['discount'];
                if ($remaining_discount != 0.00 && $most_expensive_product_pos === $j) {
                    $discount += $remaining_discount;
                    $remaining_discount = 0;
                }

                /**
                 * Product name max chars = 250, product code max chars = 50
                 * @version 3.0.1
                 */
                $product_name = $name.($discount > 0.00 ? sprintf(' (%01.2f)', $productArr['price_wt']) : '');
                $product_name = Tools::substr($product_name, 0, 250);

                $product_code = Tools::substr($productArr['reference'], 0, 50);

                /**
                 * If we maxed out our product size we push everything to one later
                 * @version 3.0.1
                 */
                if ($product_count > $cut_size - 1) {
                    $left_overs[] = [
                        'CostInclVatAmount' => max(0, ($productArr['price_wt'] - $discount)),
                        'CostVatPercent' => $productArr['rate'],
                    ];

                    continue;
                }

                $product = $xml->addChild('CofContractProduct');
                $product->addChild('Name', $product_name);
                $product->addChild('Code', $product_code);
                $product->addChild('Currency', $currency['iso_code']);
                $product->addChild('CostInclVatAmount', max(0, ($productArr['price_wt'] - $discount)));
                $product->addChild('CostVatPercent', $productArr['rate']);
            }
        }

        /**
         * Add leftovers to the end, change product name to => Other products and add up total value.
         * @version 3.0.1
         */
        if (!empty($left_overs)) {
            $product_name = $this->l('Other products');
            $product_code = '';

            $total_cost = 0;
            $rate = '';

            foreach ($left_overs as $left_over) {
                $total_cost += $left_over['CostInclVatAmount'];
                $rate = $left_over['CostVatPercent'];
            }

            $product = $xml->addChild('CofContractProduct');
            $product->addChild('Name', $product_name);
            $product->addChild('Code', $product_code);
            $product->addChild('Currency', $currency['iso_code']);
            $product->addChild('CostInclVatAmount', $total_cost);
            $product->addChild('CostVatPercent', $rate);
        }

        $shipping_cost = $this->context->cart->getTotalShippingCost();
        if ($shipping_included && $shipping_cost > 0.00) {
            $carrier = new Carrier($this->context->cart->id_carrier);
            $address = new Address($this->context->cart->id_address_delivery);

            $product = $xml->addChild('CofContractProduct');
            $product->addChild('Name', $this->l('Shipping'));
            $product->addChild('Code', $this->l('shipping'));
            $product->addChild('Currency', $currency['iso_code']);
            $product->addChild('CostInclVatAmount', $shipping_cost);
            $product->addChild('CostVatPercent', $carrier->getTaxesRate($address));
        }

        $xml->addChild('ValidToDtime', date('c', strtotime('+1 hours')));
        $domxml = dom_import_simplexml($xml);

        return $domxml->ownerDocument->saveXML($domxml->ownerDocument->documentElement);
    }

    private function getFormFields()
    {
        // $currency = CurrencyCore::getCurrency($this->context->cart->id_currency);
        $VK = array();
        $VK['VK_SERVICE'] = $this->module->getConfigFromFile($this->server, 'vk_service');
        $VK['VK_VERSION'] = $this->module->getConfigFromFile($this->server, 'vk_version');
        $VK['VK_SND_ID'] = $this->module->getConfigFromFile($this->server, 'vk_snd');
        $VK['VK_REC_ID'] = $this->module->getConfigFromFile($this->server, 'vk_rec');
        $VK['VK_STAMP'] = $this->context->cart->id;
        $VK['VK_DATA'] = (string)$this->createOrderXML();
        $VK['VK_RESPONSE'] = $this->context->link->getModuleLink($this->module->name, 'validation', array(), true);
        $VK['VK_RETURN'] = $VK['VK_RESPONSE'];
        $VK['VK_DATETIME'] = date('c');
        $VK['VK_MAC'] = $this->generateMAC($VK);
        $VK['VK_ENCODING'] = $this->module->getConfigFromFile($this->server, 'vk_encoding');
        $VK['VK_LANG'] = $this->module->getConfigFromFile($this->server, 'vk_lang');

        return $VK;
    }

    public function generateMAC(Array $source)
    {
        $data = '';
        foreach ($source as $label) {
            $data .= $this->module->padding($label).$label;
        }

        $privKey = openssl_get_privatekey(
            $this->module->getConfigFromFile($this->server, 'private_key'),
            $this->module->getConfigFromFile($this->server, 'private_key_pass')
        );

        if (!$privKey) {
            echo sprintf(
                $this->module->l(
                    'There is an error with %s Modules private key. Please contact the server administrator.',
                    'confirmation'
                ),
                $this->module->displayName
            );

            return 0;
        }

        openssl_sign($data, $signature, $privKey);
        $VK_MAC = base64_encode($signature);
        openssl_free_key($privKey);

        return $VK_MAC;
    }

    protected function l($string, $specific = false, $class = null, $addslashes = false, $htmlentities = true)
    {
        return $this->module->l($string, 'confirmation');
    }
}
